@@ -5,18 +5,56 @@ use autodie;
 
 use 5.010001;
 
-use Carp;
+use Carp qw();
 use IO::Zlib;
 use IPC::Cmd;
 use List::Util;
-use List::MoreUtils::PP;
 use Path::Tiny;
 use Tie::IxHash;
-use YAML::Syck;
+use YAML::Syck qw();
 
 use AlignDB::IntSpan;
 use App::RL::Common;
 
+sub mean {
+    @_ = grep { defined $_ } @_;
+    return 0 unless @_;
+    return $_[0] unless @_ > 1;
+    return List::Util::sum(@_) / scalar(@_);
+}
+
+sub any (&@) {
+    my $f = shift;
+    for (@_) {
+        return 1 if $f->();
+    }
+    return 0;
+}
+
+sub all (&@) {
+    my $f = shift;
+    for (@_) {
+        return 0 unless $f->();
+    }
+    return 1;
+}
+
+sub uniq (@) {
+    my %seen = ();
+    my $k;
+    my $seen_undef;
+    return grep { defined $_ ? not $seen{ $k = $_ }++ : not $seen_undef++ } @_;
+}
+
+sub firstidx (&@) {
+    my $f = shift;
+    for my $i ( 0 .. $#_ ) {
+        local *_ = \$_[$i];
+        return $i if $f->();
+    }
+    return -1;
+}
+
 sub read_replaces {
     my $file = shift;
 
@@ -179,8 +217,7 @@ sub seq_length {
 sub indel_intspan {
     my $seq = shift;
 
-    #@type AlignDB::IntSpan
-    my $intspan = AlignDB::IntSpan->new;
+    my $intspan = AlignDB::IntSpan->new();
     my $length  = length($seq);
 
     my $offset = 0;
@@ -214,8 +251,7 @@ sub indel_intspan {
 sub seq_intspan {
     my $seq = shift;
 
-    #@type AlignDB::IntSpan
-    my $intspan = AlignDB::IntSpan->new;
+    my $intspan = AlignDB::IntSpan->new();
     my $length  = length($seq);
     $intspan->add_pair( 1, $length );
 
@@ -264,17 +300,20 @@ sub align_seqs {
     }
 
     if ( !defined $bin ) {
-        confess "Could not find the executable for $aln_prog\n";
+        Carp::confess "Could not find the executable for $aln_prog\n";
     }
 
     # temp in and out
-    my $temp_in  = Path::Tiny->tempfile("seq_in_XXXXXXXX");
+    #@type Path::Tiny
+    my $temp_in = Path::Tiny->tempfile("seq_in_XXXXXXXX");
+
+    #@type Path::Tiny
     my $temp_out = Path::Tiny->tempfile("seq_out_XXXXXXXX");
 
     # msa may change the order of sequences
     my @indexes = 0 .. scalar( @{$seq_refs} - 1 );
     {
-        my $fh = $temp_in->openw;
+        my $fh = $temp_in->openw();
         for my $i (@indexes) {
             printf {$fh} ">seq_%d\n", $i;
             printf {$fh} "%s\n",      $seq_refs->[$i];
@@ -346,7 +385,7 @@ sub align_seqs_quick {
     my $seq_count = scalar @aligned;
 
     # all indel regions
-    my $realign_region = AlignDB::IntSpan->new;
+    my $realign_region = AlignDB::IntSpan->new();
     for my $seq (@aligned) {
         my $indel_intspan = indel_intspan($seq);
         $indel_intspan = $indel_intspan->pad($indel_pad);
@@ -389,7 +428,7 @@ sub trim_pure_dash {
 
     return unless $seq_length;
 
-    my $trim_region = AlignDB::IntSpan->new;
+    my $trim_region = AlignDB::IntSpan->new();
 
     for my $pos ( 1 .. $seq_length ) {
         my @bases;
@@ -398,7 +437,7 @@ sub trim_pure_dash {
             push @bases, $base;
         }
 
-        if ( List::MoreUtils::PP::all { $_ eq '-' } @bases ) {
+        if ( all { $_ eq '-' } @bases ) {
             $trim_region->add($pos);
         }
     }
@@ -440,7 +479,7 @@ sub trim_outgroup {
     my $union_set     = AlignDB::IntSpan::union(@indel_intspans);
     my $intersect_set = AlignDB::IntSpan::intersect(@indel_intspans);
 
-    my $trim_region = AlignDB::IntSpan->new;
+    my $trim_region = AlignDB::IntSpan->new();
     for my $span ( $union_set->runlists ) {
         if ( $intersect_set->superset($span) ) {
             $trim_region->add($span);
@@ -492,7 +531,7 @@ sub trim_complex_indel {
     my $union_set     = AlignDB::IntSpan::union(@indel_intspans);
     my $intersect_set = AlignDB::IntSpan::intersect(@indel_intspans);
 
-    my $complex_region = AlignDB::IntSpan->new;
+    my $complex_region = AlignDB::IntSpan->new();
     for ( reverse $intersect_set->spans ) {
         my $seg_start = $_->[0];
         my $seg_end   = $_->[1];
@@ -574,13 +613,6 @@ sub read_fasta {
     return \%seq_of;
 }
 
-sub mean {
-    @_ = grep { defined $_ } @_;
-    return 0 unless @_;
-    return $_[0] unless @_ > 1;
-    return List::Util::sum(@_) / scalar(@_);
-}
-
 sub calc_gc_ratio {
     my $seq_refs = shift;
 
@@ -710,18 +742,18 @@ sub multi_seq_stat {
             my $base = substr( $seq_refs->[$i], $pos - 1, 1 );
             push @bases, $base;
         }
-        @bases = List::MoreUtils::PP::uniq(@bases);
+        @bases = uniq(@bases);
 
-        if ( List::MoreUtils::PP::all { $_ =~ /[agct]/i } @bases ) {
+        if ( all { $_ =~ /[agct]/i } @bases ) {
             $comparable_bases++;
-            if ( List::MoreUtils::PP::all { $_ eq $bases[0] } @bases ) {
+            if ( all { $_ eq $bases[0] } @bases ) {
                 $identities++;
             }
             else {
                 $differences++;
             }
         }
-        elsif ( List::MoreUtils::PP::any { $_ eq '-' } @bases ) {
+        elsif ( any { $_ eq '-' } @bases ) {
             $gaps++;
         }
         else {
@@ -764,8 +796,8 @@ sub get_snps {
             push @bases, $base;
         }
 
-        if ( List::MoreUtils::PP::all { $_ =~ /[agct]/i } @bases ) {
-            if ( List::MoreUtils::PP::any { $_ ne $bases[0] } @bases ) {
+        if ( all { $_ =~ /[agct]/i } @bases ) {
+            if ( any { $_ ne $bases[0] } @bases ) {
                 $snp_bases_of->{$pos} = \@bases;
             }
         }
@@ -783,7 +815,7 @@ sub get_snps {
         my $mutant_to;
         my $snp_freq = 0;
         my $snp_occured;
-        my @class = List::MoreUtils::PP::uniq(@bases);
+        my @class = uniq(@bases);
         if ( scalar @class < 2 ) {
             Carp::confess "no snp\n";
         }
@@ -886,7 +918,7 @@ sub get_indels {
 
     my $seq_count = scalar @{$seq_refs};
 
-    my $indel_set = AlignDB::IntSpan->new;
+    my $indel_set = AlignDB::IntSpan->new();
     for my $i ( 0 .. $seq_count - 1 ) {
         my $seq_indel_set = indel_intspan( $seq_refs->[$i] );
         $indel_set->merge($seq_indel_set);
@@ -906,7 +938,7 @@ sub get_indels {
         my $indel_all_seqs = join "|", @indel_seqs;
 
         my $indel_type;
-        my @uniq_indel_seqs = List::MoreUtils::PP::uniq(@indel_seqs);
+        my @uniq_indel_seqs = uniq(@indel_seqs);
 
         # seqs with least '-' char wins
         my ($indel_seq) = map { $_->[0] }
@@ -989,10 +1021,10 @@ sub polarize_indel {
 
         my ( $indel_type, $indel_occured, $indel_freq );
 
-        my $indel_set = AlignDB::IntSpan->new("$site->{indel_start}-$site->{indel_end}");
+        my $indel_set = AlignDB::IntSpan->new->add_pair( $site->{indel_start}, $site->{indel_end} );
 
         # this line is different to previous subroutines
-        my @uniq_indel_seqs = List::MoreUtils::PP::uniq( @indel_seqs, $indel_outgroup_seq );
+        my @uniq_indel_seqs = uniq( @indel_seqs, $indel_outgroup_seq );
 
         # seqs with least '-' char wins
         my ($indel_seq) = map { $_->[0] }
@@ -1161,4 +1193,90 @@ sub align_to_chr {
     return $chr_pos;
 }
 
+#pod =method calc_ld
+#pod
+#pod     my ( $r, $dprime ) = App::Fasops::Common::calc_ld("111000", "111000");
+#pod
+#pod Returns the r and D' (Hill and Robertson, 1968) of two polymorphic sites.
+#pod
+#pod L<https://cran.r-project.org/web/packages/genetics/genetics.pdf>
+#pod
+#pod =cut
+
+sub calc_ld {
+    my $strA = shift;
+    my $strB = shift;
+
+    my $size = length $strA;
+
+    if ( $size != length $strB ) {
+        Carp::confess "Lengths not equal for [$strA] and [$strA]\n";
+        return;
+    }
+
+    for ( $strA, $strB ) {
+        if (/[^10]/) {
+            Carp::confess "[$_] contains illegal chars\n";
+            return;
+        }
+    }
+
+    my $A_count = $strA =~ tr/1/1/;
+    my $fA      = $A_count / $size;
+    my $fa      = 1 - $fA;
+
+    my $B_count = $strB =~ tr/1/1/;
+    my $fB      = $B_count / $size;
+    my $fb      = 1 - $fB;
+
+    if ( any { $_ == 0 } ( $fA, $fa, $fB, $fb ) ) {
+        return ( undef, undef );
+    }
+
+    # '1' in strA and '1' in strB as fAB
+    my ( $AB_count, $fAB ) = ( 0, 0 );
+    for my $i ( 1 .. $size ) {
+        my $ichar = substr $strA, $i - 1, 1;
+        my $schar = substr $strB, $i - 1, 1;
+        if ( $ichar eq '1' and $schar eq '1' ) {
+            $AB_count++;
+        }
+    }
+    $fAB = $AB_count / $size;
+
+    my $DAB = $fAB - $fA * $fB;
+
+    my ( $r, $dprime );
+    $r = $DAB / sqrt( $fA * $fa * $fB * $fb );
+
+    if ( $DAB < 0 ) {
+        $dprime = $DAB / List::Util::min( $fA * $fB, $fa * $fb );
+    }
+    else {
+        $dprime = $DAB / List::Util::min( $fA * $fb, $fa * $fB );
+    }
+
+    return ( $r, $dprime );
+}
+
 1;
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+App::Fasops::Common - collection of common subroutines
+
+=head1 SYNOPSIS
+
+    use App::Fasops::Common;
+
+    my $length = App::Fasops::Common::seq_length("AGCTTT---CCA");
+
+=head1 METHODS
+
+=cut
